=== modified file 'NEWS'
--- NEWS	2010-05-01 08:15:40 +0000
+++ NEWS	2010-05-05 18:55:10 +0000
@@ -28,6 +28,9 @@
     - Set Cache-Control and Expires headers on static pages.
       (John Arbash Meinel)
 
+    - Generate relative links where possible (everywhere but HTTP
+      redirects and feed IDs). (Michael Hudson, Matt Nordhoff)
+
 
 1.17 [20Aug2009]
 ----------------

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2010-04-14 17:54:32 +0000
+++ loggerhead/apps/branch.py	2010-05-05 18:28:54 +0000
@@ -89,10 +89,15 @@
             if v is not None:
                 qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
-        return request.construct_url(
-            self._environ, script_name=self._url_base,
-            path_info=unicode('/'.join(args)).encode('utf-8'),
-            querystring=qs)
+        path_info = urllib.quote(
+            unicode('/'.join(args)).encode('utf-8'), safe='/~:')
+        if qs:
+            path_info += '?' + qs
+        return self._url_base + path_info
+
+    def absolute_url(self, *args, **kw):
+        rel_url = self.url(*args, **kw)
+        return request.resolve_relative_url(rel_url, self._environ)
 
     def context_url(self, *args, **kw):
         kw = util.get_context(**kw)
@@ -154,7 +159,7 @@
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
-                self._url_base + '/changes')
+                self.absolute_url('/changes'))
         if path == 'static':
             return static_app(environ, start_response)
         cls = self.controllers_dict.get(path)

=== modified file 'loggerhead/apps/config.py'
--- loggerhead/apps/config.py	2009-05-13 13:14:04 +0000
+++ loggerhead/apps/config.py	2010-05-05 18:35:35 +0000
@@ -208,8 +208,8 @@
                                 environ['SCRIPT_NAME']
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             response = WSGIResponse()
             self.browse(response)

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2010-05-04 02:31:07 +0000
+++ loggerhead/apps/transport.py	2010-05-05 18:35:35 +0000
@@ -69,8 +69,8 @@
     def app_for_non_branch(self, environ):
         segment = path_info_pop(environ)
         if segment is None:
-            raise httpexceptions.HTTPMovedPermanently(
-                environ['SCRIPT_NAME'] + '/')
+            raise httpexceptions.HTTPMovedPermanently.relative_redirect(
+                environ['SCRIPT_NAME'] + '/', environ)
         elif segment == '':
             if self.name:
                 name = self.name

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2010-04-08 01:08:33 +0000
+++ loggerhead/controllers/download_ui.py	2010-05-05 18:35:35 +0000
@@ -44,8 +44,8 @@
             args.append(arg)
 
         if len(args) < 2:
-            raise httpexceptions.HTTPMovedPermanently(self._branch.url(
-                      '../changes'))
+            raise httpexceptions.HTTPMovedPermanently(
+                self._branch.absolute_url('../changes'))
 
         revid = h.fix_revid(args[0])
         file_id = args[1]

=== modified file 'loggerhead/templates/atom.pt'
--- loggerhead/templates/atom.pt	2009-06-06 12:23:11 +0000
+++ loggerhead/templates/atom.pt	2010-05-05 18:42:58 +0000
@@ -4,8 +4,8 @@
     bazaar changes for <tal:branch-name content="branch/friendly_name">branch name</tal:branch-name>
   </title>
   <updated tal:content="string:${updated}Z">${updated}</updated>
-  <id tal:content="python:branch.url(['/atom'])">url</id>
-  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.url(['/atom'])" />
+  <id tal:content="python:branch.absolute_url(['/atom'])">url</id>
+  <link rel="self" type="application/atom+xml" tal:attributes="href python:branch.absolute_url(['/atom'])" />
   <link rel="alternate" type="text/html" tal:attributes="href python:branch.url(['/changes'])" />
 
   <entry tal:repeat="entry changes">
@@ -16,7 +16,7 @@
       updated
     </updated>
     <!-- TODO: The feed validator will generate warnings because this is URL-encoded -->
-    <id tal:content="python:branch.url(['/revision', entry.revno])">
+    <id tal:content="python:branch.absolute_url(['/revision', entry.revno])">
       ID
     </id>
     <author tal:repeat="author entry/authors">

